@@ -37,12 +37,12 @@
  *
  * Last reviewed on 2013-07-11 (1.0.0)
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "rtsp-media-factory.h"
 
-#define GST_RTSP_MEDIA_FACTORY_GET_PRIVATE(obj)  \
-       (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_RTSP_MEDIA_FACTORY, GstRTSPMediaFactoryPrivate))
-
 #define GST_RTSP_MEDIA_FACTORY_GET_LOCK(f)       (&(GST_RTSP_MEDIA_FACTORY_CAST(f)->priv->lock))
 #define GST_RTSP_MEDIA_FACTORY_LOCK(f)           (g_mutex_lock(GST_RTSP_MEDIA_FACTORY_GET_LOCK(f)))
 #define GST_RTSP_MEDIA_FACTORY_UNLOCK(f)         (g_mutex_unlock(GST_RTSP_MEDIA_FACTORY_GET_LOCK(f)))
@@ -62,9 +62,12 @@ struct _GstRTSPMediaFactoryPrivate
   GstRTSPTransportMode transport_mode;
   gboolean stop_on_disconnect;
   gchar *multicast_iface;
+  guint max_mcast_ttl;
+  gboolean bind_mcast_address;
 
   GstClockTime rtx_time;
   guint latency;
+  gboolean do_retransmission;
 
   GMutex medias_lock;
   GHashTable *medias;           /* protected by medias_lock */
@@ -85,8 +88,11 @@ struct _GstRTSPMediaFactoryPrivate
                                         GST_RTSP_LOWER_TRANS_TCP
 #define DEFAULT_BUFFER_SIZE     0x80000
 #define DEFAULT_LATENCY         200
+#define DEFAULT_MAX_MCAST_TTL   255
+#define DEFAULT_BIND_MCAST_ADDRESS FALSE
 #define DEFAULT_TRANSPORT_MODE  GST_RTSP_TRANSPORT_MODE_PLAY
 #define DEFAULT_STOP_ON_DISCONNECT TRUE
+#define DEFAULT_DO_RETRANSMISSION FALSE
 
 enum
 {
@@ -102,6 +108,8 @@ enum
   PROP_TRANSPORT_MODE,
   PROP_STOP_ON_DISCONNECT,
   PROP_CLOCK,
+  PROP_MAX_MCAST_TTL,
+  PROP_BIND_MCAST_ADDRESS,
   PROP_LAST
 };
 
@@ -134,15 +142,14 @@ static void default_configure (GstRTSPMediaFactory * factory,
 static GstElement *default_create_pipeline (GstRTSPMediaFactory * factory,
     GstRTSPMedia * media);
 
-G_DEFINE_TYPE (GstRTSPMediaFactory, gst_rtsp_media_factory, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (GstRTSPMediaFactory, gst_rtsp_media_factory,
+    G_TYPE_OBJECT);
 
 static void
 gst_rtsp_media_factory_class_init (GstRTSPMediaFactoryClass * klass)
 {
   GObjectClass *gobject_class;
 
-  g_type_class_add_private (klass, sizeof (GstRTSPMediaFactoryPrivate));
-
   gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->get_property = gst_rtsp_media_factory_get_property;
@@ -224,6 +231,19 @@ gst_rtsp_media_factory_class_init (GstRTSPMediaFactoryClass * klass)
           "medias of this factory", GST_TYPE_CLOCK,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_MAX_MCAST_TTL,
+      g_param_spec_uint ("max-mcast-ttl", "Maximum multicast ttl",
+          "The maximum time-to-live value of outgoing multicast packets", 1,
+          255, DEFAULT_MAX_MCAST_TTL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property (gobject_class, PROP_BIND_MCAST_ADDRESS,
+      g_param_spec_boolean ("bind-mcast-address", "Bind mcast address",
+          "Whether the multicast sockets should be bound to multicast addresses "
+          "or INADDR_ANY",
+          DEFAULT_BIND_MCAST_ADDRESS,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_rtsp_media_factory_signals[SIGNAL_MEDIA_CONSTRUCTED] =
       g_signal_new ("media-constructed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRTSPMediaFactoryClass,
@@ -250,7 +270,7 @@ static void
 gst_rtsp_media_factory_init (GstRTSPMediaFactory * factory)
 {
   GstRTSPMediaFactoryPrivate *priv =
-      GST_RTSP_MEDIA_FACTORY_GET_PRIVATE (factory);
+      gst_rtsp_media_factory_get_instance_private (factory);
   factory->priv = priv;
 
   priv->launch = g_strdup (DEFAULT_LAUNCH);
@@ -264,6 +284,9 @@ gst_rtsp_media_factory_init (GstRTSPMediaFactory * factory)
   priv->transport_mode = DEFAULT_TRANSPORT_MODE;
   priv->stop_on_disconnect = DEFAULT_STOP_ON_DISCONNECT;
   priv->publish_clock_mode = GST_RTSP_PUBLISH_CLOCK_MODE_CLOCK;
+  priv->do_retransmission = DEFAULT_DO_RETRANSMISSION;
+  priv->max_mcast_ttl = DEFAULT_MAX_MCAST_TTL;
+  priv->bind_mcast_address = DEFAULT_BIND_MCAST_ADDRESS;
 
   g_mutex_init (&priv->lock);
   g_mutex_init (&priv->medias_lock);
@@ -338,6 +361,14 @@ gst_rtsp_media_factory_get_property (GObject * object, guint propid,
     case PROP_CLOCK:
       g_value_take_object (value, gst_rtsp_media_factory_get_clock (factory));
       break;
+    case PROP_MAX_MCAST_TTL:
+      g_value_set_uint (value,
+          gst_rtsp_media_factory_get_max_mcast_ttl (factory));
+      break;
+    case PROP_BIND_MCAST_ADDRESS:
+      g_value_set_boolean (value,
+          gst_rtsp_media_factory_is_bind_mcast_address (factory));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, propid, pspec);
   }
@@ -388,6 +419,14 @@ gst_rtsp_media_factory_set_property (GObject * object, guint propid,
     case PROP_CLOCK:
       gst_rtsp_media_factory_set_clock (factory, g_value_get_object (value));
       break;
+    case PROP_MAX_MCAST_TTL:
+      gst_rtsp_media_factory_set_max_mcast_ttl (factory,
+          g_value_get_uint (value));
+      break;
+    case PROP_BIND_MCAST_ADDRESS:
+      gst_rtsp_media_factory_set_bind_mcast_address (factory,
+          g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, propid, pspec);
   }
@@ -1087,6 +1126,55 @@ gst_rtsp_media_factory_get_retransmission_time (GstRTSPMediaFactory * factory)
   return res;
 }
 
+/**
+ * gst_rtsp_media_factory_set_do_retransmission:
+ *
+ * Set whether retransmission requests will be sent for
+ * receiving media
+ *
+ * Since: 1.16
+ */
+void
+gst_rtsp_media_factory_set_do_retransmission (GstRTSPMediaFactory * factory,
+    gboolean do_retransmission)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+
+  g_return_if_fail (GST_IS_RTSP_MEDIA_FACTORY (factory));
+
+  priv = factory->priv;
+
+  GST_DEBUG_OBJECT (factory, "Do retransmission %d", do_retransmission);
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  priv->do_retransmission = do_retransmission;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+}
+
+/**
+ * gst_rtsp_media_factory_get_do_retransmission:
+ *
+ * Returns: Whether retransmission requests will be sent for receiving media
+ *
+ * Since: 1.16
+ */
+gboolean
+gst_rtsp_media_factory_get_do_retransmission (GstRTSPMediaFactory * factory)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+  gboolean res;
+
+  g_return_val_if_fail (GST_IS_RTSP_MEDIA_FACTORY (factory), 0);
+
+  priv = factory->priv;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  res = priv->do_retransmission;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+
+  return res;
+}
+
 /**
  * gst_rtsp_media_factory_set_latency:
  * @factory: a #GstRTSPMediaFactory
@@ -1415,6 +1503,110 @@ gst_rtsp_media_factory_get_publish_clock_mode (GstRTSPMediaFactory * factory)
   return ret;
 }
 
+/**
+ * gst_rtsp_media_factory_set_max_mcast_ttl:
+ * @factory: a #GstRTSPMedia
+ * @ttl: the new multicast ttl value
+ *
+ * Set the maximum time-to-live value of outgoing multicast packets.
+ *
+ * Returns: %TRUE if the requested ttl has been set successfully.
+ */
+gboolean
+gst_rtsp_media_factory_set_max_mcast_ttl (GstRTSPMediaFactory * factory,
+    guint ttl)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+
+  g_return_val_if_fail (GST_IS_RTSP_MEDIA_FACTORY (factory), FALSE);
+
+  priv = factory->priv;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  if (ttl == 0 || ttl > DEFAULT_MAX_MCAST_TTL) {
+    GST_WARNING_OBJECT (factory, "The requested mcast TTL value is not valid.");
+    GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+    return FALSE;
+  }
+  priv->max_mcast_ttl = ttl;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+
+  return TRUE;
+}
+
+/**
+ * gst_rtsp_media_factory_get_max_mcast_ttl:
+ * @factory: a #GstRTSPMedia
+ *
+ * Get the the maximum time-to-live value of outgoing multicast packets.
+ *
+ * Returns: the maximum time-to-live value of outgoing multicast packets.
+ */
+guint
+gst_rtsp_media_factory_get_max_mcast_ttl (GstRTSPMediaFactory * factory)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+  guint result;
+
+  g_return_val_if_fail (GST_IS_RTSP_MEDIA_FACTORY (factory), 0);
+
+  priv = factory->priv;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  result = priv->max_mcast_ttl;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+
+  return result;
+}
+
+/**
+ * gst_rtsp_media_factory_set_bind_mcast_address:
+ * @factory: a #GstRTSPMediaFactory
+ * @bind_mcast_addr: the new value
+ *
+ * Decide whether the multicast socket should be bound to a multicast address or
+ * INADDR_ANY.
+ */
+void
+gst_rtsp_media_factory_set_bind_mcast_address (GstRTSPMediaFactory * factory,
+    gboolean bind_mcast_addr)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+
+  g_return_if_fail (GST_IS_RTSP_MEDIA_FACTORY (factory));
+
+  priv = factory->priv;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  priv->bind_mcast_address = bind_mcast_addr;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+}
+
+/**
+ * gst_rtsp_media_factory_is_bind_mcast_address:
+ * @factory: a #GstRTSPMediaFactory
+ *
+ * Check if multicast sockets are configured to be bound to multicast addresses.
+ *
+ * Returns: %TRUE if multicast sockets are configured to be bound to multicast addresses.
+ */
+gboolean
+gst_rtsp_media_factory_is_bind_mcast_address (GstRTSPMediaFactory * factory)
+{
+  GstRTSPMediaFactoryPrivate *priv;
+  gboolean result;
+
+  g_return_val_if_fail (GST_IS_RTSP_MEDIA_FACTORY (factory), FALSE);
+
+  priv = factory->priv;
+
+  GST_RTSP_MEDIA_FACTORY_LOCK (factory);
+  result = priv->bind_mcast_address;
+  GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
+
+  return result;
+}
+
 static gchar *
 default_gen_key (GstRTSPMediaFactory * factory, const GstRTSPUrl * url)
 {
@@ -1565,6 +1757,8 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   GstClock *clock;
   gchar *multicast_iface;
   GstRTSPPublishClockMode publish_clock_mode;
+  guint ttl;
+  gboolean bind_mcast;
 
   /* configure the sharedness */
   GST_RTSP_MEDIA_FACTORY_LOCK (factory);
@@ -1580,6 +1774,8 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   stop_on_disconnect = priv->stop_on_disconnect;
   clock = priv->clock ? gst_object_ref (priv->clock) : NULL;
   publish_clock_mode = priv->publish_clock_mode;
+  ttl = priv->max_mcast_ttl;
+  bind_mcast = priv->bind_mcast_address;
   GST_RTSP_MEDIA_FACTORY_UNLOCK (factory);
 
   gst_rtsp_media_set_suspend_mode (media, suspend_mode);
@@ -1589,10 +1785,13 @@ default_configure (GstRTSPMediaFactory * factory, GstRTSPMedia * media)
   gst_rtsp_media_set_profiles (media, profiles);
   gst_rtsp_media_set_protocols (media, protocols);
   gst_rtsp_media_set_retransmission_time (media, rtx_time);
+  gst_rtsp_media_set_do_retransmission (media, priv->do_retransmission);
   gst_rtsp_media_set_latency (media, latency);
   gst_rtsp_media_set_transport_mode (media, transport_mode);
   gst_rtsp_media_set_stop_on_disconnect (media, stop_on_disconnect);
   gst_rtsp_media_set_publish_clock_mode (media, publish_clock_mode);
+  gst_rtsp_media_set_max_mcast_ttl (media, ttl);
+  gst_rtsp_media_set_bind_mcast_address (media, bind_mcast);
 
   if (clock) {
     gst_rtsp_media_set_clock (media, clock);
