@@ -34,14 +34,14 @@
  *
  * Last reviewed on 2013-07-16 (1.0.0)
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 
 #include "rtsp-session.h"
 
-#define GST_RTSP_SESSION_MEDIA_GET_PRIVATE(obj)  \
-    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_RTSP_SESSION_MEDIA, GstRTSPSessionMediaPrivate))
-
 struct _GstRTSPSessionMediaPrivate
 {
   GMutex lock;
@@ -65,15 +65,14 @@ GST_DEBUG_CATEGORY_STATIC (rtsp_session_media_debug);
 
 static void gst_rtsp_session_media_finalize (GObject * obj);
 
-G_DEFINE_TYPE (GstRTSPSessionMedia, gst_rtsp_session_media, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (GstRTSPSessionMedia, gst_rtsp_session_media,
+    G_TYPE_OBJECT);
 
 static void
 gst_rtsp_session_media_class_init (GstRTSPSessionMediaClass * klass)
 {
   GObjectClass *gobject_class;
 
-  g_type_class_add_private (klass, sizeof (GstRTSPSessionMediaPrivate));
-
   gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize = gst_rtsp_session_media_finalize;
@@ -85,9 +84,9 @@ gst_rtsp_session_media_class_init (GstRTSPSessionMediaClass * klass)
 static void
 gst_rtsp_session_media_init (GstRTSPSessionMedia * media)
 {
-  GstRTSPSessionMediaPrivate *priv = GST_RTSP_SESSION_MEDIA_GET_PRIVATE (media);
+  GstRTSPSessionMediaPrivate *priv;
 
-  media->priv = priv;
+  media->priv = priv = gst_rtsp_session_media_get_instance_private (media);
 
   g_mutex_init (&priv->lock);
   priv->state = GST_RTSP_STATE_INIT;
@@ -281,6 +280,8 @@ gst_rtsp_session_media_get_rtpinfo (GstRTSPSessionMedia * media)
     }
 
     stream = gst_rtsp_stream_transport_get_stream (transport);
+    if (!gst_rtsp_stream_is_sender (stream))
+      continue;
     if (!gst_rtsp_stream_get_rtpinfo (stream, NULL, NULL, NULL, &running_time))
       continue;
 
@@ -415,8 +416,6 @@ gst_rtsp_session_media_get_transport (GstRTSPSessionMedia * media, guint idx)
  *
  * Returns: (transfer full) (element-type GstRTSPStreamTransport): a
  * list of #GstRTSPStreamTransport, g_ptr_array_unref () after usage.
- *
- * Since: 1.14
  */
 GPtrArray *
 gst_rtsp_session_media_get_transports (GstRTSPSessionMedia * media)
